#!/usr/bin/env ruby
#
# Check systemd services
# ===
#
# Examples:
#
#    # neutron-server service is running
#    check-service -s neutron-server
#
#    # nova-network service is NOT running
#    check-service --no-running -s neutron-server
#
# Copyright 2015 x-ion GmbH, <info@x-ion.de>
#
# Released under the same terms as Sensu (the MIT license); see LICENSE
# for details.

require 'rubygems' if RUBY_VERSION < '1.9.0'
require 'sensu-plugin/check/cli'
require 'timeout'

class CheckService < Sensu::Plugin::Check::CLI

  option :running,
    :short => '-y',
    :long => '--[no-]running',
    :description => 'Should the service be running?',
    :boolean => true,
    :default => true

  option :name,
    :short => '-s NAME',
    :long => '--service NAME',
    :description => 'The name of the service.'

  option :timeout,
    :description => 'Timeout (default 10)',
    :short => '-t SEC',
    :long => '--timeout',
    :proc => proc { |t| t.to_i },
    :default => 10

  def run_cmd(cmd)
    begin
      output, status = nil
      Timeout.timeout(config[:timeout]) do
        output = `#{cmd}`
        status = $?.success?
      end
    rescue => error
      critical "Error: #{error.to_s}"
    end
    status
  end

  def run
    # Force systemd to create unit for service. For is-active and is-failed,
    # systemd assumes "inactive" if it cannot find the unit in its memory
    # (e.g., after a daemon-reload).
    `systemctl show #{config[:name]} -p ActiveState`
    failed = run_cmd "systemctl is-failed #{config[:name]}"
    critical "Service #{config[:name]} is failed" if failed

    active = run_cmd "systemctl is-active #{config[:name]}"
    case active
      when false
        ok "Service #{config[:name]} is in-active" if not config[:running]
        critical "Service #{config[:name]} is in-active"
      when true
        ok "Service #{config[:name]} is active" if config[:running]
        critical "Service #{config[:name]} is active"
    end
  end

end
