#!/usr/bin/env ruby
#
# Directory Usage Metrics
# ========
#
# This plugin uses `du` to collect disk usage metrics.
#
# Directories can be specifically ignored using -i option.  Pass --sudo to make
# sensu use `sudo` for the `du` call.
#
# Copyright 2014 x-ion GmbH
#
# Released under the same terms as Sensu (the MIT license); see LICENSE
# for details.
#
# rubocop:disable HandleExceptions

require 'rubygems' if RUBY_VERSION < '1.9.0'
require 'sensu-plugin/metric/cli'
require 'timeout'
require 'socket'

class DirUsageMetrics < Sensu::Plugin::Metric::CLI::Graphite

  option :scheme,
         :description => 'Metric naming scheme, text to prepend to .$parent.$child',
         :long => '--scheme SCHEME',
         :default => "#{Socket.gethostname}.dir_usage"

  option :timeout,
         :description => 'Timeout (default 10)',
         :short => '-t SEC',
         :long => '--timeout',
         :proc => proc { |t| t.to_i },
         :default => 10

  option :dir,
         :description => 'Meter directory usage',
         :short => '-d DIR',
         :long => '--dir',
         :required => true

  option :ignore,
         :description => 'Regular expressions to ignore',
         :short => '-i RE[,RE]',
         :long => '--ignore',
         :proc => proc { |a| a.split(',').map {|re| Regexp.new(re) }}

  option :sudo,
         :description => 'Use sudo',
         :long => '--sudo',
         :boolean => true,
         :default => false

  def run_cmd(cmd)
    begin
      output, status = nil
      Timeout.timeout(config[:timeout]) do
        c = config[:sudo] ? 'sudo ' : ''
        c += cmd
        output = `#{c}`
        status = $?.success?
      end
    rescue => error
      critical "Error: #{error.to_s}"
    end
    critical "Command '#{c}' returned no output" if output == ''
    critical output unless status
    output
  end

  def run
    data = run_cmd("du #{config[:dir]}")

    data.lines.each do |line|
      size, name = line.split(/\t/).map(&:strip)

      next if config[:ignore] && config[:ignore].find{ |re| re.match name }

      var_name = "#{config[:scheme]}.#{name[1..-1].gsub(/\.|\//,'_')}"
      output var_name, size
    end

    ok
  end
end
